use Test::More tests => 1;
BEGIN { use_ok('Lingua::DxExtractor') };

#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.


